@@ -59,9 +59,9 @@ import operator
 import re
 import contextlib
 import io
-import distutils.dir_util
 import importlib
 import itertools
+import platform
 
 try:
     import win32security
@@ -95,6 +95,7 @@ if PY2:
     getcwdu = os.getcwdu
     map = itertools.imap
     FileNotFoundError = OSError
+    itertools.filterfalse = itertools.ifilterfalse
 
 
 @contextlib.contextmanager
@@ -1411,34 +1412,60 @@ class Path(text_type):
 
     cd = chdir
 
-    def merge_tree(self, dst, symlinks=False, *args, **kwargs):
+    def merge_tree(
+            self, dst, symlinks=False,
+            # *
+            update=False,
+            copy_function=shutil.copy2,
+            ignore=lambda dir, contents: []):
         """
         Copy entire contents of self to dst, overwriting existing
         contents in dst with those in self.
 
-        If the additional keyword `update` is True, each
-        `src` will only be copied if `dst` does not exist,
-        or `src` is newer than `dst`.
+        Pass ``symlinks=True`` to copy symbolic links as links.
 
-        Note that the technique employed stages the files in a temporary
-        directory first, so this function is not suitable for merging
-        trees with large files, especially if the temporary directory
-        is not capable of storing a copy of the entire source tree.
+        Accepts a ``copy_function``, similar to copytree.
+
+        To avoid overwriting newer files, supply a copy function
+        wrapped in ``only_newer``. For example::
+
+            src.merge_tree(dst, copy_function=only_newer(shutil.copy2))
         """
-        update = kwargs.pop('update', False)
-        with TempDir() as _temp_dir:
-            # first copy the tree to a stage directory to support
-            #  the parameters and behavior of copytree.
-            stage = _temp_dir / str(hash(self))
-            self.copytree(stage, symlinks, *args, **kwargs)
-            # now copy everything from the stage directory using
-            #  the semantics of dir_util.copy_tree
-            distutils.dir_util.copy_tree(
-                stage,
-                dst,
-                preserve_symlinks=symlinks,
-                update=update,
+        dst = self._next_class(dst)
+        dst.makedirs_p()
+
+        if update:
+            warnings.warn(
+                "Update is deprecated; "
+                "use copy_function=only_newer(shutil.copy2)",
+                DeprecationWarning,
+                stacklevel=2,
             )
+            copy_function = only_newer(copy_function)
+
+        sources = self.listdir()
+        _ignored = ignore(self, [item.name for item in sources])
+
+        def ignored(item):
+            return item.name in _ignored
+
+        for source in itertools.filterfalse(ignored, sources):
+            dest = dst / source.name
+            if symlinks and source.islink():
+                target = source.readlink()
+                target.symlink(dest)
+            elif source.isdir():
+                source.merge_tree(
+                    dest,
+                    symlinks=symlinks,
+                    update=update,
+                    copy_function=copy_function,
+                    ignore=ignore,
+                )
+            else:
+                copy_function(source, dest)
+
+        self.copystat(dst)
 
     #
     # --- Special stuff from os
@@ -1573,6 +1600,23 @@ class Path(text_type):
         return functools.partial(SpecialResolver, cls)
 
 
+def only_newer(copy_func):
+    """
+    Wrap a copy function (like shutil.copy2) to return
+    the dst if it's newer than the source.
+    """
+    @functools.wraps(copy_func)
+    def wrapper(src, dst, *args, **kwargs):
+        is_newer_dst = (
+            dst.exists()
+            and dst.getmtime() >= src.getmtime()
+        )
+        if is_newer_dst:
+            return dst
+        return copy_func(src, dst, *args, **kwargs)
+    return wrapper
+
+
 class SpecialResolver(object):
     class ResolverScope:
         def __init__(self, paths, scope):
@@ -1961,3 +2005,38 @@ class FastPath(Path):
 
         pattern, normcase = self.__prepare(pattern, normcase)
         return self.__fnmatch(pattern, normcase)
+
+
+def patch_for_linux_python2():
+    """
+    As reported in #130, when Linux users create filenames
+    not in the file system encoding, it creates problems on
+    Python 2. This function attempts to patch the os module
+    to make it behave more like that on Python 3.
+    """
+    if not PY2 or platform.system() != 'Linux':
+        return
+
+    try:
+        import backports.os
+    except ImportError:
+        return
+
+    class OS:
+        """
+        The proxy to the os module
+        """
+        def __init__(self, wrapped):
+            self._orig = wrapped
+
+        def __getattr__(self, name):
+            return getattr(self._orig, name)
+
+        def listdir(self, *args, **kwargs):
+            items = self._orig.listdir(*args, **kwargs)
+            return list(map(backports.os.fsdecode, items))
+
+    globals().update(os=OS(os))
+
+
+patch_for_linux_python2()
